package eu.jonahbauer.chat.bot.database.impl;

import eu.jonahbauer.chat.bot.database.BindException;
import eu.jonahbauer.chat.bot.database.impl.types.TypeAdapter;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Constructor;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

final class BinderFactory extends AbstractFactory<Binder<?>> {
    public static final @NotNull BinderFactory INSTANCE = new BinderFactory();

    private BinderFactory() {}

    @Override
    @SneakyThrows
    protected @NotNull <T extends Record> Binder<T> computeValue0(@NotNull Class<T> type) {
        var components = type.getRecordComponents();

        var columns = new ArrayList<String>();
        var adapters = new ArrayList<TypeAdapter<?>>();
        var types = new ArrayList<Class<?>>();

        for (var component : components) {
            var componentType = component.getType();
            types.add(componentType);
            columns.add(Helper.getColumnName(component));
            adapters.add(TypeAdapter.get(componentType));
        }

        var ctor = type.getDeclaredConstructor(types.toArray(Class[]::new));
        return new BinderImpl<>(ctor, columns, adapters);
    }

    private static final class BinderImpl<T extends Record> implements Binder<T> {
        private final @NotNull Constructor<T> constructor;
        private final @NotNull List<@NotNull String> columns;
        private final @NotNull List<@NotNull TypeAdapter<?>> adapters;

        private BinderImpl(@NotNull Constructor<T> constructor, @NotNull List<@NotNull String> columns, @NotNull List<@NotNull TypeAdapter<?>> adapters) {
            this.constructor = Objects.requireNonNull(constructor);
            this.columns = List.copyOf(columns);
            this.adapters = List.copyOf(adapters);
            if (this.columns.size() != this.adapters.size()) throw new IllegalArgumentException();
        }

        @Override
        public @NotNull T bind(@NotNull ResultSet result, @NotNull String prefix) throws SQLException {
            try {
                var count = columns.size();
                var arguments = new Object[count];
                for (int i = 0; i < count; i++) {
                    var index = result.findColumn(prefix + columns.get(i));
                    arguments[i] = adapters.get(i).extract(result, index);
                }
                return constructor.newInstance(arguments);
            } catch (Exception ex) {
                throw new BindException(ex);
            }
        }
    }
}
