package eu.jonahbauer.chat.bot.database.impl.types;

import org.jetbrains.annotations.NotNull;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.time.Instant;

public class InstantTypeAdapter implements TypeAdapter<Instant> {

    @Override
    public void prepare(@NotNull PreparedStatement stmt, int index, Instant value) throws SQLException {
        if (value == null) {
            stmt.setNull(index, Types.INTEGER);
        } else {
            stmt.setLong(index, value.toEpochMilli());
        }
    }

    @Override
    public Instant extract(@NotNull ResultSet result, int index) throws SQLException {
        var millis = result.getLong(index);
        if (result.wasNull()) {
            return null;
        } else {
            return Instant.ofEpochMilli(millis);
        }
    }
}
