package eu.jonahbauer.chat.bot.database.impl.types;

import org.jetbrains.annotations.NotNull;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

public class JdbcTypeAdapter<T> implements TypeAdapter<T> {
    private final @NotNull Class<T> type;

    JdbcTypeAdapter(@NotNull Class<T> type) {
        this.type = type;
    }

    @Override
    public void prepare(@NotNull PreparedStatement stmt, int index, T value) throws SQLException {
        if (type == byte.class || type == Byte.class) {
            if (value == null) stmt.setNull(index, Types.INTEGER);
            else stmt.setByte(index, (byte) value);
        } else if (type == char.class || type == Character.class) {
            if (value == null) stmt.setNull(index, Types.INTEGER);
            else stmt.setShort(index, (short) (char) value);
        } else if (type == short.class || type == Short.class) {
            if (value == null) stmt.setNull(index, Types.INTEGER);
            else stmt.setShort(index, (short) value);
        } else if (type == int.class || type == Integer.class) {
            if (value == null) stmt.setNull(index, Types.INTEGER);
            else stmt.setInt(index, (int) value);
        } else if (type == long.class || type == Long.class) {
            if (value == null) stmt.setNull(index, Types.INTEGER);
            else stmt.setLong(index, (long) value);
        } else if (type == float.class || type == Float.class) {
            if (value == null) stmt.setNull(index, Types.FLOAT);
            else stmt.setFloat(index, (float) value);
        } else if (type == double.class || type == Double.class) {
            if (value == null) stmt.setNull(index, Types.DOUBLE);
            else stmt.setDouble(index, (double) value);
        } else if (type == boolean.class || type == Boolean.class) {
            if (value == null) stmt.setNull(index, Types.BOOLEAN);
            else stmt.setBoolean(index, (boolean) value);
        } else {
            stmt.setObject(index, value);
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    public T extract(@NotNull ResultSet result, int index) throws SQLException {
        if (type == byte.class || type == Byte.class) {
            var out = (T) (Byte) result.getByte(index);
            return result.wasNull() ? null : out;
        } else if (type == char.class || type == Character.class) {
            var out = (T) (Character) (char) result.getShort(index);
            return result.wasNull() ? null : out;
        } else if (type == short.class || type == Short.class) {
            var out = (T) (Short) result.getShort(index);
            return result.wasNull() ? null : out;
        } else if (type == int.class || type == Integer.class) {
            var out = (T) (Integer) result.getInt(index);
            return result.wasNull() ? null : out;
        } else if (type == long.class || type == Long.class) {
            var out = (T) (Long) result.getLong(index);
            return result.wasNull() ? null : out;
        } else if (type == float.class || type == Float.class) {
            var out = (T) (Float) result.getFloat(index);
            return result.wasNull() ? null : out;
        } else if (type == double.class || type == Double.class) {
            var out = (T) (Double) result.getDouble(index);
            return result.wasNull() ? null : out;
        } else if (type == boolean.class || type == Boolean.class) {
            var out = (T) (Boolean) result.getBoolean(index);
            return result.wasNull() ? null : out;
        } else {
            return result.getObject(index, type);
        }
    }
}
