package eu.jonahbauer.chat.bot.database.impl.types;

import org.jetbrains.annotations.NotNull;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.time.Instant;

public interface TypeAdapter<T> {
    @SuppressWarnings("unchecked")
    static <T> @NotNull TypeAdapter<T> get(@NotNull Class<T> type) {
        if (type == Instant.class) {
            return (TypeAdapter<T>) new InstantTypeAdapter();
        } else if (Enum.class.isAssignableFrom(type)) {
            return (TypeAdapter<T>) newEnumTypeAdapter(type);
        } else {
            return new JdbcTypeAdapter<>(type);
        }
    }

    @SuppressWarnings("unchecked")
    private static <E extends Enum<E>> EnumTypeAdapter<?> newEnumTypeAdapter(Class<?> type) {
        return new EnumTypeAdapter<>((Class<E>) type);
    }

    @SuppressWarnings("unchecked")
    default void prepareUnsafe(@NotNull PreparedStatement stmt, int index, Object value) throws SQLException {
        prepare(stmt, index, (T) value);
    }

    void prepare(@NotNull PreparedStatement stmt, int index, T value) throws SQLException;

    T extract(@NotNull ResultSet result, int index) throws SQLException;

}
