package eu.jonahbauer.chat.server.management;

import eu.jonahbauer.chat.server.management.annotations.ManagedAttribute;
import eu.jonahbauer.chat.server.management.annotations.ManagedOperation;
import eu.jonahbauer.chat.server.management.annotations.ManagedParameter;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;

import javax.management.MXBean;
import javax.management.ObjectName;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;

@SuppressWarnings("unused")
@MXBean
public interface ChatBotManagerMXBean {
    @NotNull ObjectName NAME = getObjectName();

    @SneakyThrows
    private static @NotNull ObjectName getObjectName() {
        return ObjectName.getInstance("eu.jonahbauer.chat.bot.server", "component", "ChatBots");
    }

    @ManagedOperation(description = "starts a bot with the given name and type", impact = ManagedOperation.Impact.ACTION)
    default void start(
            @ManagedParameter(name = "name", description = "the name of the bot (must be unique)") @NotNull String name,
            @ManagedParameter(name = "type", description = "the fully qualified class name of the bot") @NotNull String type,
            @ManagedParameter(name = "channel", description = "the channel the bot will be active in") @NotNull String channel
    ) {
        start(name, type, List.of(channel));
    }

    @ManagedOperation(description = "starts a bot with the given name and type", impact = ManagedOperation.Impact.ACTION)
    void start(
            @ManagedParameter(name = "name", description = "the name of the bot (must be unique)") @NotNull String name,
            @ManagedParameter(name = "type", description = "the fully qualified class name of the bot") @NotNull String type,
            @ManagedParameter(name = "channels", description = "the channels the bot will be active in") @NotNull List<@NotNull String> channels
    );

    @ManagedOperation(description = "starts a bot with the given config under the specified name", impact = ManagedOperation.Impact.ACTION)
    void start(
            @ManagedParameter(name = "name", description = "the name of the bot (must be unique)") @NotNull String name,
            @ManagedParameter(name = "config", description = "the bot configuration") @NotNull BotConfigSupport config
    );

    @ManagedOperation(description = "stops the bot with the given name and waits for it to finish", impact = ManagedOperation.Impact.ACTION)
    void stop(@ManagedParameter(name = "name", description = "the name of the bot") @NotNull String name) throws InterruptedException;

    @ManagedOperation(description = "stops all currently running bots and waits for them to finish", impact = ManagedOperation.Impact.ACTION)
    void stop() throws InterruptedException;

    @ManagedAttribute(description = "the list of all currently running bots.")
    @NotNull Map<@NotNull String, @NotNull BotConfigSupport> getBots();

    @ManagedAttribute(description = "the list of all known bot implementations")
    @NotNull SortedSet<@NotNull String> getBotImplementations();
}
