package eu.jonahbauer.chat.server.json;

import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;

public class BooleanTypeAdapter extends TypeAdapter<Boolean> {
    @Override
    public void write(@NotNull JsonWriter out, @Nullable Boolean value) throws IOException {
        if (value == null) {
            out.nullValue();
        } else {
            out.value(value ? 1 : 0);
        }
    }

    @Override
    public Boolean read(@NotNull JsonReader in) throws IOException {
        return switch (in.peek()) {
            case NUMBER -> in.nextInt() != 0;
            case BOOLEAN -> in.nextBoolean();
            case NULL -> null;
            default -> throw new IllegalStateException("Expected a number but was " + in.peek());
        };
    }
}
