package eu.jonahbauer.chat.server.management.impl;

import eu.jonahbauer.chat.server.management.SocketManagerMXBean;
import eu.jonahbauer.chat.server.socket.SocketSupervisor;
import org.jetbrains.annotations.NotNull;

import javax.management.JMException;
import java.lang.management.ManagementFactory;
import java.util.Objects;
import java.util.SortedSet;

public final class SocketManager implements SocketManagerMXBean {
    private final @NotNull SocketSupervisor supervisor;

    public static void init(@NotNull SocketSupervisor supervisor) throws JMException {
        var impl = new SocketManager(supervisor);
        var server = ManagementFactory.getPlatformMBeanServer();
        server.registerMBean(new AdvancedMBean(impl), SocketManagerMXBean.NAME);
    }

    private SocketManager(@NotNull SocketSupervisor supervisor) {
        this.supervisor = Objects.requireNonNull(supervisor);
    }

    @Override
    public void setCredentials(@NotNull String username, @NotNull String password) {
        supervisor.setAccount(username, password);
    }

    @Override
    public void start(@NotNull String channel) {
        supervisor.start(channel);
    }

    @Override
    public void stop(@NotNull String channel) throws InterruptedException {
        supervisor.stop(channel);
    }

    @Override
    public void stop() throws InterruptedException {
        supervisor.stop();
    }

    @Override
    public @NotNull SortedSet<@NotNull String> getChannels() {
        return supervisor.getChannels();
    }
}
