package eu.jonahbauer.chat.server.management.impl;

import eu.jonahbauer.chat.server.management.SocketState;
import eu.jonahbauer.chat.server.management.SocketSupportMXBean;
import eu.jonahbauer.chat.server.socket.SocketSupervisor;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.lang.management.ManagementFactory;
import java.util.Date;
import java.util.Objects;

@Slf4j
@Getter
public final class SocketSupport implements SocketSupportMXBean {
    private final @NotNull SocketSupervisor supervisor;
    private final @NotNull String channel;

    public static void register(@NotNull SocketSupervisor supervisor, @NotNull String channel) {
        try {
            var impl = new SocketSupport(supervisor, channel);
            var server = ManagementFactory.getPlatformMBeanServer();
            server.registerMBean(new AdvancedMBean(impl), SocketSupportMXBean.getObjectName(channel));
        } catch (Exception ex) {
            log.error("Could not register socket as an MBean.", ex);
        }
    }

    public static void unregister(@NotNull String name) {
        try {
            var server = ManagementFactory.getPlatformMBeanServer();
            server.unregisterMBean(SocketSupportMXBean.getObjectName(name));
        } catch (Exception ex) {
            log.error("Could not unregister socket as an MBean.", ex);
        }
    }

    private SocketSupport(@NotNull SocketSupervisor supervisor, @NotNull String channel) {
        this.supervisor = Objects.requireNonNull(supervisor);
        this.channel = Objects.requireNonNull(channel);
    }

    @Override
    public @Nullable Date getCooldownUntil() {
        var cooldown = supervisor.getCooldownUntil(channel);
        return cooldown != null ? Date.from(cooldown) : null;
    }

    @Override
    public @Nullable SocketState getState() {
        return supervisor.getState(channel);
    }


    @Override
    public void stop() throws InterruptedException {
        supervisor.stop(channel);
    }

    @Override
    public void restart() {
        supervisor.restart(channel);
    }

    @Override
    public void send(@NotNull String name, @NotNull String message, boolean bottag, boolean publicid) {
        supervisor.send(channel, name, message, bottag, publicid);
    }
}
