package eu.jonahbauer.chat.server.util;

import org.jetbrains.annotations.NotNull;

import java.util.Objects;

public interface Lazy<T> {
    @NotNull T get();

    class MutableLazy<T> implements Lazy<T> {
        private T value;

        @Override
        public @NotNull T get() {
            var value = this.value;
            if (value == null) throw new IllegalStateException();
            return value;
        }

        public @NotNull T set(@NotNull T value) {
            if (this.value != null) throw new IllegalStateException();
            this.value = Objects.requireNonNull(value);
            return value;
        }
    }
}
